class Utils(object):

    @staticmethod
    def remove_tags(html):
        """
        Return a new string without html tags.

        >>> remove_tags("<a href='#'>Something</a>")
        'Something'
        """
        import re
        return re.sub('<[^>]*>', ' ', html).strip()

    @staticmethod
    def text_before(text, after):
        """
        Return text before after.

        >>> text_before("<a href='#'>Something</a>", "</a>")
        "<a href='#'>Something"
        """
        index = text.find(after)
        if index > -1:
            return text[:index]
        return text

    @staticmethod
    def text_after(text, before):
        """
        Return text after before.

        >>> text_after("<a href='#'>Something</a>", "<a href='#'>")
        'Something</a>'
        """
        index = text.find(before)
        if index > -1:
            index += len(before)
            return text[index:]
        return text

    @staticmethod
    def text_between(text, before, after, force_html=False, second=False):
        """
        Return text between before and after.
        Use force_html when before and after were html tags.

        >>> text_between("<a href='#'>Something</a>", "<a href='#'>", "</a>")
        'Something'
        """
        if not second:
            start = text.find(before)
        else:
            first = text.find(before)
            second = text[first + 1:].find(before)
            start = first + second

        if start > -1:
            start += len(before)
        if force_html:
            if before[-1] != ">":
                start = text.find(">", start) + 1
        end = text.find(after, start)
        if force_html:
            if after[0] != "<":
                end = text.find("<", start)
        if -1 < start < end:
            return text[start:end]
        return text

    @staticmethod
    def remove_spaces(text):
        """
        Return a new string without double space, tabs, carriage return
        or line feed.

        >>> remove_spaces("Something  else")
        'Something else'
        """
        text = text.replace("\t", " ")
        text = text.replace("\n", " ")
        text = text.replace("\r", " ")
        while text.find("  ") > -1:
            text = text.replace("  ", " ")
        return text.strip()

    @staticmethod
    def remove_accents(text):
        """
        Return a new string without accents from portuguese

        >>> remove_accents("trava-língua")
        'trava-lingua'
        """
        reference = [
            ('a', 'áàâãä'),
            ('e', 'éèêë'),
            ('i', 'íìîï'),
            ('o', 'óòôõö'),
            ('u', 'úùûü'),
            ('c', 'ç')
        ]

        new_text = ""
        for char in text:
            found = False
            for clear_vowal, possible_accents in reference:
                if char in possible_accents:
                    new_text += clear_vowal
                    found = True
                    break
            if not found:
                new_text += char
        return new_text

    @staticmethod
    def split_html_tag(text, tag):
        """
        Return a list like split, but it uses html tags in various formats.

        >>> str = "Something<br>else<br />and<br/>another<br></br>thing"
        >>> split_html_tag(str, "br")
        ['Something', 'else', 'and', 'another', 'thing']
        """
        import re
        expression = '<[^>]*{0}[^>]*>'.format(tag)
        return list(filter(None, re.split(expression, text)))



"""
Unofficial Python API for Dicio.com.br

@author Felipe Pontes
@email felipemfpontes@gmail.com
"""

import html
from urllib.request import urlopen
from dicio.utils import Utils

BASE_URL = 'http://www.dicio.com.br/{}'
CHARSET = 'utf-8'
TAG_MEANING = ('class="significado', '</p>')
TAG_ETYMOLOGY = ('class="etim', '</span>')
TAG_SYNONYMS = ('class="adicional sinonimos"', '</p>')
TAG_SYNONYMS_DELIMITER = ('<a', '</a>')
TAG_EXTRA = ('class="adicional"', '</p>')
TAG_EXTRA_SEP = 'br'
TAG_EXTRA_DELIMITER = ('<b>', '</b>')
TAG_PHRASE_DELIMITER = ('<div class="frase"', '</div>')


class Word(object):

    def __init__(self, word, meaning=None, etymology=None, synonyms=[], antonyms=[], examples=[], extra={}):
        self.word = word.strip().lower()
        self.url = BASE_URL.format(Utils.remove_accents(self.word))
        self.meaning = meaning
        self.etymology = etymology
        self.synonyms = synonyms
        self.antonyms = antonyms
        self.extra = extra
        self.examples = examples

    def load(self, dicio=None, get=urlopen):
        if dicio:
            found = dicio.search(self.word)
        else:
            found = Dicio(get).search(self.word)

        if found is not None:
            self.word = found.word
            self.meaning = found.meaning
            self.etymology = found.etymology
            self.synonyms = found.synonyms
            self.antonyms = found.antonyms
            self.extra = found.extra
            self.examples = found.examples

    def __repr__(self):
        return 'Word({!r})'.format(self.word)

    def __str__(self):
        if self.meaning:
            return self.word + ': ' + self.meaning
        return self.word


class Dicio(object):
    """
    Dicio API with meaning, synonyms and extra information.
    """

    def __init__(self, get=urlopen):
        self.get = get

    def search(self, word):
        """
        Search for word.
        """
        if len(word.split()) > 1:
            return None

        _word = Utils.remove_accents(word).strip().lower()
        try:
            with self.get(BASE_URL.format(_word)) as request:
                page = html.unescape(request.read().decode(CHARSET))
        except:
            return None

        meaning, etymology = self.scrape_meaning(page)
        synonyms, antonyms = self.scrape_synonyms(page)
        return Word(
            Utils.text_between(page, "<h1", "</h1>",  force_html=True).lower(),
            meaning=meaning,
            etymology=etymology,
            synonyms=synonyms,
            antonyms=antonyms,
            examples=self.scrape_examples(page),
            extra=self.scrape_extra(page),
        )

    def scrape_meaning(self, page):
        """
        Return meaning and etymology.
        """
        html = Utils.text_between(page, *TAG_MEANING, force_html=True)

        etymology = Utils.text_between(html, *TAG_ETYMOLOGY, force_html=True)
        etymology = Utils.remove_spaces(Utils.remove_tags(etymology))

        meanings = Utils.split_html_tag(html, 'br')
        meanings = [Utils.remove_spaces(Utils.remove_tags(x))
                    for x in meanings]
        meaning = '; '.join([x for x in meanings if x != etymology])

        return meaning, etymology

    def scrape_synonyms(self, page):
        """
        Return list of synonyms.
        """
        synonyms = []
        antonyms = []
        first_synonyms_position = page.find(TAG_SYNONYMS[0])
        if first_synonyms_position > -1:
            html = Utils.text_between(page, *TAG_SYNONYMS, force_html=True)
            while html.find(TAG_SYNONYMS_DELIMITER[0]) > -1:
                synonym, html = self.first_synonym(html)
                synonyms.append(synonym)
            second_synonyms_position = \
                page[first_synonyms_position+1:].find(TAG_SYNONYMS[0])
            if second_synonyms_position > -1:
                html = Utils.text_between(page, *TAG_SYNONYMS,
                                          force_html=True, second=True)
                while html.find(TAG_SYNONYMS_DELIMITER[0]) > -1:
                    antonym, html = self.first_synonym(html)
                    antonyms.append(antonym)
        return synonyms, antonyms

    def first_synonym(self, html):
        """
        Return the first synonym found and html without his marking.
        """
        synonym = Utils.text_between(html, *TAG_SYNONYMS_DELIMITER,
                                     force_html=True)
        synonym = Utils.remove_spaces(synonym)
        _html = html.replace(TAG_SYNONYMS_DELIMITER[0], "", 1)
        _html = _html.replace(TAG_SYNONYMS_DELIMITER[1], "", 1)
        return Word(synonym), _html

    def scrape_examples(self, page):
        """
        Return a list of examples.
        """
        examples = []
        html = page
        index = html.find(TAG_PHRASE_DELIMITER[0])
        while index > -1:
            example_html = Utils.text_between(
                html, *TAG_PHRASE_DELIMITER, force_html=True)
            examples += [Utils.remove_spaces(Utils.remove_tags(example_html))]
            html = html[index+len(TAG_PHRASE_DELIMITER[0]):]
            index = html.find(TAG_PHRASE_DELIMITER[0])
        return examples

    def scrape_extra(self, page):
        """
        Return a dictionary of extra information.
        """
        dict_extra = {}
        try:
            if page.find(TAG_EXTRA[0]) > -1:
                html = Utils.text_between(page, *TAG_EXTRA, force_html=True)
                extra_rows = Utils.split_html_tag(Utils.remove_spaces(html),
                                                  TAG_EXTRA_SEP)
                for row in extra_rows:
                    _row = Utils.remove_tags(row)
                    key, value = map(Utils.remove_spaces, _row.split(":"))
                    dict_extra[key] = value
        except:
            pass
        return dict_extra



import pathlib

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name='dicio_scrapper',
    version='0.0.1',
    author='Data & Analytics Research',
    author_email='analytics.dar@take.net',
    packages=['dicio'],
    test_suite='tests',
    url='https://github.com/miloutsch/dicio',
    license='MIT License',
    description='Unofficial Python API for Dicio based on the original API of Felipe Pontes.',
    credits=['Felipe Pontes', 'Ramon Durães', 'Milo Utsch'],
    long_description=README,
    long_description_content_type="text/markdown",
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Internet :: WWW/HTTP :: Browsers',
        'Topic :: Internet :: WWW/HTTP :: Indexing/Search',
        'Topic :: Utilities',
    ],
)
